C ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

      SUBROUTINE rjfpkvalm (crec, irval, iival, idbg, ierr)

C @(#)Subroutine: rjfpkvalm          Author: R.A. Jansen -- Aug 27, 2009
C @(#)
C @(#)  Purpose:  Write a complex integer-valued keyword value to a FITS
C @(#)    header record 'crec' for a keyword value of which the real and
C @(#)    imaginary parts are given by 'irval' and 'iival' that are both
C @(#)    declared as variables of the default INTEGER data type. Depen-
C @(#)    ding on the host OS and compiler, this may accomodate a 16-bit
C @(#)    (INTEGER*2)  but more generally  a 32-bit (INTEGER*4)  integer.
C @(#)    The keyword value will be written as a comma-separated pair of
C @(#)    integers, enclosed in parentheses,  and left-justified on byte
C @(#)    11 in the header record.  If verbosity level 'idbg' > 0, addi-
C @(#)    tional messages may be printed.  Should any error condition be
C @(#)    detected, error status 'ierr' will be non-zero upon exit.  Its
C @(#)    value (if >2000) uniquely identifies a known error condition.
C @(#)
C @(#)  Calling sequence:
C @(#)    CALL rjfpkvalm(crec, irval, iival, idbg, ierr) ,  where 'crec'
C @(#)    is a character string with a declared length of  80 characters
C @(#)    and  'irval' and 'iival'  are integers of the default  INTEGER 
C @(#)    data type. 'idbg' and 'ierr' are both integers.
C @(#)
C @(#)  Note:  this task is meant to write a complex keyword value  to a
C @(#)    header record that has been initialized with the FITS keyword;
C @(#)    use one of 'rjfmkval?' to modify an existing keyword value.
C @(#)
C @(#)  Dependencies:	clen, cinit
C @(#)
C @(#)  Revision history:
C @(#)    Aug 27, 2009 [RAJ] -- Initial version of this routine.
C @(#)    Sep 13, 2009 [RAJ] -- Changed error code numbers.
C @(#)------------------------------------------------------------------

      IMPLICIT NONE

C --- Declaration of input/return variables...
      CHARACTER	crec*80
      INTEGER	irval, iival, idbg, ierr

C --- Declaration of local variables...
      CHARACTER dbid*19, ctmp*80, cval*80, cc
      INTEGER	clen, ic, i, j, irtmp, iitmp

C     'i0','i9','ib','ip', and 'im' are shorthand integer equivalents
C     in the ASCII collating sequence of digits 0 and 9, space, + and -,
C     respectively...
      INTEGER	ib, i0, i9, ip, im

C     ------------------------------------------------------------------

C --- Return for non-zero input error status...
      IF (ierr.NE.0) RETURN

C --- Initialize the debugging ID string...
      dbid  = 'DEBUG(rjfpkvalm): '

C --- Initialize local variables...
      CALL cinit(cval)
      CALL cinit(ctmp)
      ib  = ICHAR(' ')
      i0  = ICHAR('0')
      i9  = ICHAR('9')
      ip  = ICHAR('+')
      im  = ICHAR('-')

C --- Try to write the real and imaginary parts of the complex keyword
C     value to temporary character string 'ctmp'...
      WRITE(ctmp,'(A1,SPI30,A1,SPI30,A1)') '(',irval,',',iival,')'

C --- Copy the non-blank characters to character string 'cval', left-
C     justified on byte 11, while testing on-the-fly that only digits,
C     plus, minus, comma and parentheses were present...
      i = 10
      DO j=1,clen(ctmp)
         cc = ctmp(j:j)
         ic = ICHAR(cc)
         IF (ic.NE.ib) THEN
            IF ((ic.GE.i0.AND.ic.LE.i9).OR.ic.EQ.ip.OR.ic.EQ.im
     +          .OR.ic.EQ.40.OR.ic.EQ.41.OR.ic.EQ.44) THEN
               i = i + 1
               cval(i:i) = cc
            ELSE
C              Write appears to have failed (likely ***, or some such)..
               IF (idbg.GT.0) WRITE(*,*) dbid//
     +           'failed to write COMPLEX INT keyword value to record.'
               ierr = 2431
               GOTO 9
            ENDIF
         ENDIF
      ENDDO
      i = i + 1
      cval(i:i) = ' '

C --- Try to read back the value written and compare with the original..
      CALL cinit(ctmp)
      ctmp = cval
C     First, replace parentheses and comma by a blank..
      DO i=1,clen(ctmp)
         cc = ctmp(i:i)
         ic = ICHAR(cc)
         IF (ic.EQ.40.OR.ic.EQ.41.OR.ic.EQ.44) ctmp(i:i) = ' '
      ENDDO
C     Then try and read back real and imaginary parts as two integers...
      READ(ctmp,*,IOSTAT=ierr) irtmp, iitmp
      IF (ierr.NE.0) THEN
         IF (idbg.GT.0) WRITE(*,*) dbid//
     +     'failed to read back COMPLEX INT keyword value from record.'
         ierr = 2432
         GOTO 9
      ENDIF
      IF (irtmp.NE.irval) THEN
         IF (idbg.GT.0) WRITE(*,*) dbid//'real part of complex '//
     +      'keyword value read back from record does not match '//
     +      'input value.'
         ierr = 2433
         GOTO 9
      ENDIF
      IF (iitmp.NE.iival) THEN
         IF (idbg.GT.0) WRITE(*,*) dbid//'imaginary part of complex '//
     +      'keyword value read back from record does not match '//
     +      'input value.'
         ierr = 2434
         GOTO 9
      ENDIF


C --- We appear to have successfully written the complex integer keyword
C     value, so copy characters 11 and on to the output header record...
      j = clen(cval)
      crec(11:j) = cval(11:j)


9     IF (idbg.GT.2) WRITE(*,*) dbid//crec(1:clen(crec))
      IF (idbg.GT.1) WRITE(*,*) dbid//'on exit, ierr = ',ierr

      RETURN
      END


