C ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

      SUBROUTINE rjfpkvalq (crec, drval, dival, idbg, ierr)

C @(#)Subroutine: rjfpkvalq          Author: R.A. Jansen -- Sep  2, 2009
C @(#)
C @(#)  Purpose:  Write a complex double precision-valued  keyword value
C @(#)    to a  FITS header record  'crec'  for a keyword value of which
C @(#)    the real and imaginary parts are given by 'drval' and 'dival',
C @(#)    both declared as variables of the  REAL*8 data type.  For most
C @(#)    host OS  and  compiler implementations,  this corresponds to a
C @(#)    64-bit floating point real.  The keyword value will be written
C @(#)    using default formatting  as a comma-separated pair of  double
C @(#)    precision reals,  enclosed in parentheses,  and left-justified
C @(#)    on byte 11 in the header record.  If verbosity level 'idbg'>0,
C @(#)    additional messages may be printed.   Error status 'ierr' will
C @(#)    be non-zero upon exit, should any error condition be detected.
C @(#)    Its value (if >2000) identifies a known error condition.
C @(#)
C @(#)  Calling sequence:
C @(#)    CALL rjfpkvalq(crec, drval, dival, idbg, ierr) ,  where 'crec'
C @(#)    is a character string with a declared length of  80 characters
C @(#)    and 'drval' and 'dival' are variables of the REAL*8 data type.
C @(#)    'idbg' and 'ierr' are both integers.
C @(#)
C @(#)  Note1: 64-bit floating point  REAL*8  values generally represent
C @(#)    at most  13 or 14 significant digits faithfully;  keep this in
C @(#)    mind when using this task.
C @(#)  Note2: internally, scaling offset 1P is enforced for exponential
C @(#)    notation, such that a value of pi is written as  3.1415927E+00
C @(#)    instead of the Fortran default 0.31415927E+01.
C @(#)  Note3: this task is meant to write a complex keyword value  to a
C @(#)    header record that has been initialized with the FITS keyword;
C @(#)    use one of 'rjfmkval?' to modify an existing keyword value.
C @(#)
C @(#)  Dependencies:	clen, cinit
C @(#)
C @(#)  Revision history:
C @(#)    Sep  2, 2009 [RAJ] -- Initial version of this routine.
C @(#)------------------------------------------------------------------

      IMPLICIT NONE

C --- Declaration of input/return variables...
      CHARACTER	crec*80
      REAL*8	drval, dival
      INTEGER	idbg, ierr

C --- Declaration of local variables...
      CHARACTER dbid*19, ctmp*80, cval*80, cfmt*36, cc
      REAL*8	drtmp, ditmp
      INTEGER	clen, ic, i, j

C     The following variables are shorthand integer equivalents in the
C     ASCII collating sequence of the digits 0 and 9, and the space, E,
C     +, -, and . , respectively...
      INTEGER	i0, i9, ib, iEE, ip, im, id

C     ------------------------------------------------------------------

C --- Return for non-zero input error status...
      IF (ierr.NE.0) RETURN

C --- Initialize the debugging ID string...
      dbid  = 'DEBUG(rjfpkvalq): '

C --- Initialize local variables...
      CALL cinit(cval)
      CALL cinit(ctmp)
      CALL cinit(cfmt)
      i0  = ICHAR('0')
      i9  = ICHAR('9')
      ib  = ICHAR(' ')
      iEE = ICHAR('E')
      ip  = ICHAR('+')
      im  = ICHAR('-')
      id  = ICHAR('.')
      drtmp = DABS(drval)
      ditmp = DABS(dival)

C --- Build format specification to use for default formatting...
      cfmt(1:4) = '(A1,'
      i = clen(cfmt)
      IF (drtmp.GT.1.0D-5.AND.drtmp.LT.1.0D5) THEN
         IF (drtmp.LT.1.0D-4) THEN
            cfmt(i+1:i+12) = 'SPF20.16,A1,'
         ELSE IF (drtmp.LT.1.0D-3) THEN
            cfmt(i+1:i+12) = 'SPF20.15,A1,'
         ELSE IF (drtmp.LT.1.0D-2) THEN
            cfmt(i+1:i+12) = 'SPF20.15,A1,'
         ELSE IF (drtmp.LT.1.0D-1) THEN
            cfmt(i+1:i+12) = 'SPF20.14,A1,'
         ELSE IF (drtmp.LT.1.0D1) THEN
            cfmt(i+1:i+12) = 'SPF20.14,A1,'
         ELSE IF (drtmp.LT.1.0D2) THEN
            cfmt(i+1:i+12) = 'SPF20.13,A1,'
         ELSE IF (drtmp.LT.1.0D3) THEN
            cfmt(i+1:i+12) = 'SPF20.13,A1,'
         ELSE IF (drtmp.LT.1.0D4) THEN
            cfmt(i+1:i+12) = 'SPF20.12,A1,'
         ELSE
            cfmt(i+1:i+12) = 'SPF20.12,A1,'
         ENDIF
      ELSE IF (drtmp.LT.1.0D100) THEN
         cfmt(i+1:i+16) = '1PSPE20.13E2,A1,'
      ELSE
         cfmt(i+1:i+16) = '1PSPE20.13E3,A1,'
      ENDIF
      i = clen(cfmt)

      IF (ditmp.GT.1.0D-5.AND.ditmp.LT.1.0D5) THEN
         IF (ditmp.LT.1.0D-4) THEN
            cfmt(i+1:i+12) = 'SPF20.16,A1)'
         ELSE IF (ditmp.LT.1.0D-3) THEN
            cfmt(i+1:i+12) = 'SPF20.15,A1)'
         ELSE IF (ditmp.LT.1.0D-2) THEN
            cfmt(i+1:i+12) = 'SPF20.15,A1)'
         ELSE IF (ditmp.LT.1.0D-1) THEN
            cfmt(i+1:i+12) = 'SPF20.14,A1)'
         ELSE IF (ditmp.LT.1.0D1) THEN
            cfmt(i+1:i+12) = 'SPF20.14,A1)'
         ELSE IF (ditmp.LT.1.0D2) THEN
            cfmt(i+1:i+12) = 'SPF20.13,A1)'
         ELSE IF (ditmp.LT.1.0D3) THEN
            cfmt(i+1:i+12) = 'SPF20.13,A1)'
         ELSE IF (ditmp.LT.1.0D4) THEN
            cfmt(i+1:i+12) = 'SPF20.12,A1)'
         ELSE
            cfmt(i+1:i+12) = 'SPF20.12,A1)'
         ENDIF
      ELSE IF (ditmp.LT.1.0D100) THEN
         cfmt(i+1:i+16) = '1PSPE20.13E2,A1)'
      ELSE
         cfmt(i+1:i+16) = '1PSPE20.13E3,A1)'
      ENDIF


C --- Try to write the real and imaginary parts of the complex keyword
C     value to temporary character string 'ctmp' using the format string
C     we just constructed...
      WRITE(ctmp,cfmt) '(',drval,',',dival,')'


C --- Copy the non-blank characters to character string 'cval', left-
C     justified on byte 11, while testing on-the-fly that only digits,
C     upper case E, plus, minus, comma and parentheses were present...
      i = 10
      DO j=1,clen(ctmp)
         cc = ctmp(j:j)
         ic = ICHAR(cc)
         IF (ic.NE.ib) THEN
            IF ((ic.GE.i0.AND.ic.LE.i9).OR.ic.EQ.id.OR.ic.EQ.iEE
     +          .OR.ic.EQ.ip.OR.ic.EQ.im.OR.ic.EQ.40.OR.ic.EQ.41
     +          .OR.ic.EQ.44) THEN
               i = i + 1
               cval(i:i) = cc
            ELSE
C              Write appears to have failed (likely ***, or some such)..
               IF (idbg.GT.0) WRITE(*,*) dbid//
     +           'failed to write COMPLEX DBLE keyword value to record.'
               ierr = 2451
               GOTO 9
            ENDIF
         ENDIF
      ENDDO
      i = i + 1
      cval(i:i) = ' '


C --- Try to read back the value written and compare with the original..
      CALL cinit(ctmp)
      ctmp = cval
      drtmp = 9.999999999D99
      ditmp = -9.999999999D99
C     First, replace parentheses and comma by a blank..
      DO i=1,clen(ctmp)
         cc = ctmp(i:i)
         ic = ICHAR(cc)
         IF (ic.EQ.40.OR.ic.EQ.41.OR.ic.EQ.44) ctmp(i:i) = ' '
      ENDDO
C     Then try and read back real and imaginary parts as two reals...
      READ(ctmp,*,IOSTAT=ierr) drtmp, ditmp
      IF (ierr.NE.0) THEN
         IF (idbg.GT.0) WRITE(*,*) dbid//
     +     'failed to read back COMPLEX DBLE keyword value from record.'
         ierr = 2452
         GOTO 9
      ENDIF
      IF (1.0d-13.LT.(DABS(drtmp-drval)/drval)) THEN
         IF (idbg.GT.0) WRITE(*,*) dbid//'real part of complex '//
     +      'keyword value read back from record does not match '//
     +      'input value.'
         ierr = 2453
         GOTO 9
      ENDIF
      IF (1.0d-13.LT.(DABS(ditmp-dival)/dival)) THEN
         IF (idbg.GT.0) WRITE(*,*) dbid//'imaginary part of complex '//
     +      'keyword value read back from record does not match '//
     +      'input value.'
         ierr = 2454
         GOTO 9
      ENDIF


C --- We appear to have successfully written the complex real keyword
C     value, so copy characters 11 and on to the output header record...
      j = clen(cval)
      crec(11:j) = cval(11:j)


9     IF (idbg.GT.2) WRITE(*,*) dbid//crec(1:clen(crec))
      IF (idbg.GT.1) WRITE(*,*) dbid//'on exit, ierr = ',ierr

      RETURN
      END


